"""Constants shared by client and server."""

# Protocol version
PROTOCOL_VERSION = "1.0"

# Maximum message size in bytes
MAX_MESSAGE_SIZE = 4096

# Heartbeat interval in seconds
HEARTBEAT_INTERVAL = 30

# Default port
DEFAULT_PORT = 9999
