"""Constants used by the client portion of the bridge"""

# Reconnection delay in seconds
RECONNECT_DELAY = 5

# Default update rate in Hz
DEFAULT_UPDATE_RATE = 60

# Default joystick index
DEFAULT_JOYSTICK_INDEX = 0
